C     
C     Last update: 11.08.1994 by Thomas Gau
C                                Technical University Braunschweig
C
      PROGRAM CUTGEN1
      PARAMETER (NMAX=100)
      DIMENSION RAND(NMAX)
      INTEGER   SEED,Z,DBAR,OL(NMAX),OQ(NMAX)
      READ (5,*) M,L,V1,V2,DBAR,SEED,ICPRB
      Z=SEED
      OPEN(9,FILE='SDAT')
      DO 10 I=1,ICPRB
         CALL CUTGEN1(M,L,V1,V2,DBAR,Z,NOL,OL,OQ,RAND)
         CALL OUTPUT(NOL,L,OL,OQ,I)
   10 CONTINUE
      CLOSE(9)
      STOP
      END         

      SUBROUTINE OUTPUT(NOL,L,OL,OQ,K)
C      
C     ******************************************************************
C     ***                                                            ***
C     *** Ausdruck der just generierten Problemdaten in die Sammel-  ***
C     *** datei SDAT (Kanal 9)                                       ***
C     ***                                                            ***
C     ******************************************************************
C
      INTEGER   OL(NOL),OQ(NOL)
C
      WRITE (9,1000) K
 1000 FORMAT('''','TEST',I4.4,'''')                   
      WRITE (9,1001) NOL
 1001 FORMAT(I8)      
      WRITE (9,1002) L
 1002 FORMAT(I8)
      DO 10 I=1,NOL
         WRITE (9,1003) OL(I),OQ(I)
   10 CONTINUE
 1003 FORMAT(I8,2X,I8)  
C 
      RETURN
      END

      SUBROUTINE CUTGEN1(M,L,V1,V2,DBAR,Z,NOL,OL,OQ,RAND)
C
C     ******************************************************************
C     ***               Problem Generator for 1D-CSP                 ***
C     ***         (One-dimensional Cutting Stock Problem)            ***
C     ******************************************************************
C
C     The input parameter must satisfy the conditions
C
C     (1) V1*L .GE. 1
C     (2) V2 .LE. 1    
C     (3) V1 .LT. V2
C     (4) M,L,DBAR,Z  positive integers
C
C     The program is completely self-contained and communication to it 
C     is achieved solely through the parameter list of CUTGEN1.
C
C     CUTGEN1 needs 3 arrays (OL,OQ,RAND) of length at least M (array
C     RAND is dummy).
C
C     Meaning of the input parameters:
C
C     M      : problem size
C     L      : size of standard length
C     V1     : lower bound for the the relative size of order lengths in
C              relation to L 
C     V2     : upper bound for the the relative size of order lengths in
C              relation to L 
C     DBAR   : average demand per order length
C     Z      : random number; when calling CUTGEN1 the first time Z will
C              be initialized by the chosen SEED
C  
C     Meaning of the output parameters:
C 
C     NOL    : number of different order lengths
C     OL(I)  : order length I (I=1,...,NOL)
C     OQ(I)  : order quantity of length I (I=1,...,NOL)
C     Z      : random number
C
C     ******************************************************************
C
      INTEGER DBAR,Z,D,REST,OL(M),OQ(M)
      REAL    RAND(M)
C
C     1. Determination of Order Lengths
      DO 10 I=1,M
         ZZZ=RAN0(Z)
         DUM=(V1+(V2-V1)*ZZZ)*L + ZZZ
         OL(I)=-INT(DUM)
   10 CONTINUE
C     
C     Sorting of lengths OL in descending order
      CALL HPSORT(M,OL)
C
C     2. Determination of Demands
      SUM=0.
      DO 20 I=1,M
         RAND(I)=RAN0(Z)
         SUM=SUM+RAND(I)
   20 CONTINUE
      D=DBAR*M     
      REST=D
      DO 30 I=1,M-1
         DUM=(RAND(I)/SUM)*D + 0.5
         OQ(I)=MAX(1,INT(DUM))
         REST=REST-OQ(I)
   30 CONTINUE         
      OQ(M)=MAX(1,REST)            
C
C     Demands of identical order lengths are summed up
      OL(1)=-OL(1)
      NOL=1
      DO 40 I=2,M
         IF (ABS(OL(I)) .NE. ABS(OL(I-1))) THEN
            NOL=NOL+1
            OL(NOL)=-OL(I)
            OQ(NOL)=OQ(I)
         ELSE   
            OQ(NOL)=OQ(NOL)+OQ(I)
         ENDIF
   40 CONTINUE        
C    
      RETURN
      END
      
      FUNCTION ran0(z)
C     Copr. 1986-92 Numerical Recipes Software '!5.
C     ******************************************************************
C     ***          Determination of Uniform Random Numbers           ***
C     ******************************************************************
C      
      INTEGER z,IA,IM,IQ,IR
      REAL ran0,AM
      PARAMETER (IA=16807,IM=2147483647,AM=1./IM,IQ=127773,IR=2836)
      INTEGER k
      k=z/IQ
      z=IA*(z-k*IQ)-IR*k
      if (z.LT.0) z=z+IM
      ran0=AM*z
      return
      END

      SUBROUTINE hpsort(n,ra)
C     Copr. 1986-92 Numerical Recipes Software '!5.
C     ******************************************************************
C     ***                Sorting Routine "Heapsort"                  ***
C     ******************************************************************
C
      INTEGER ra(n),rra
      INTEGER i,ir,j,l
      if (n.lt.2) return
      l=n/2+1
      ir=n
10    continue
        if(l.gt.1)then
          l=l-1
          rra=ra(l)
        else
          rra=ra(ir)
          ra(ir)=ra(1)
          ir=ir-1
          if(ir.eq.1)then
            ra(1)=rra
            return
          endif
        endif
        i=l
        j=l+l
20      if(j.le.ir)then
          if(j.lt.ir)then
            if(ra(j).lt.ra(j+1))j=j+1
          endif
          if(rra.lt.ra(j))then
            ra(i)=ra(j)
            i=j
            j=j+j
          else
            j=ir+1
          endif
        goto 20
        endif
        ra(i)=rra
      goto 10
      END
